
div.imagegallery div.section {
  display:inline;
 }

table.album td {
    padding-bottom: 1em;
}

li.screenshot {
  text-align:center;
  list-style-type:none;
  background:none;
}

img.selectedimg{
 border: 1px solid #FF0000;
}
